#!/usr/bin/env python
"""SNAPbuild exe wrapper"""

import logging
import os
import platform
import subprocess
import sys

from distutils.sysconfig import get_python_lib

################################################################
# Are we snapbuild (*nix) or snapbuild.exe (Win)?
EXT = ''
if platform.system().lower().startswith('win'):
    EXT = '.exe'

EXE_NAME = 'snapbuild' + EXT

LOG = logging.getLogger(__name__)

SNAPBUILD_PATH = os.path.join(os.path.dirname(os.path.abspath(__file__)), EXE_NAME)


def executing_in_virtualenv():
    """Check if function is being executed in a virtual environment.

    Returns:
        bool, if the function is executed in a virtualenv it will return True, False otherwise
    """
    return 'VIRTUAL_ENV' in os.environ


def get_virtualenv_path():
    """Get the path to the virtual environment Python library

    Returns:
        str, the path to the virtual environment Python library
    """
    return os.path.abspath(get_python_lib(prefix=os.environ['VIRTUAL_ENV']))


def try_get_site_packages():
    """Attempt to get additional site packages via site.getsitepackages.

    Needed to run on Ubuntu outside a virtualenv, because Ubuntu
    installs stuff to different directories...
    """

    try:
        import site
        return site.getsitepackages()
    except ImportError:
        LOG.info("No site package available, not trying to add site packages to includes.")
    except:
        LOG.warning("Unable to get site packages, not trying to add site packages to includes.")

    return []


def call(args=None, stderr=None):
    """Wrapper for SNAPbuild that includes site-pkgs by default
    Have to wrap it because pulling site-packages from within
    the executable would get the wrong site-packages...
    """

    if args is None:
        args = sys.argv

    addl_args = []

    # If we're called from a VIRTUAL_ENV, include that virtual env's lib
    if executing_in_virtualenv():
        addl_args += ['-I', '%s' % get_virtualenv_path()]

    # Include the lib from where we're installed
    addl_args += ['-I', '%s' % get_python_lib()]

    # Try to also include site.getsitepackages, because Ubuntu uses different paths...
    for path in try_get_site_packages():
        addl_args += ['-I', path]

    call_args = [SNAPBUILD_PATH] + args[1:] + addl_args

    return subprocess.check_output(call_args, stderr=stderr)


def _write_output(output):
    if len(output) > 0:
        for line in output.split('\n'):
            print line


def main(args=None):
    """Wrapper for SNAPbuild that includes site-pkgs by default
    Have to wrap it because pulling site-packages from within
    the executable would get the wrong site-packages...
    """

    logging.basicConfig(format='%(asctime)-15s %(levelname)-8s %(name)-8s %(message)s')

    try:
        output = call(args)
        _write_output(output)
    except subprocess.CalledProcessError as e:
        _write_output(e.output)
        exit(e.returncode)
    except:
        exit(1)
    else:
        exit(0)

if __name__ == '__main__':
    main(sys.argv)
